@@ -33,14 +33,14 @@ struct _GdkDrawableClass
 		          GdkGCValuesMask mask);
   void (*draw_rectangle) (GdkDrawable  *drawable,
 			  GdkGC	       *gc,
-			  gint		filled,
+			  gboolean	filled,
 			  gint		x,
 			  gint		y,
 			  gint		width,
 			  gint		height);
   void (*draw_arc)       (GdkDrawable  *drawable,
 			  GdkGC	       *gc,
-			  gint		filled,
+			  gboolean	filled,
 			  gint		x,
 			  gint		y,
 			  gint		width,
@@ -49,7 +49,7 @@ struct _GdkDrawableClass
 			  gint		angle2);
   void (*draw_polygon)   (GdkDrawable  *drawable,
 			  GdkGC	       *gc,
-			  gint		filled,
+			  gboolean	filled,
 			  GdkPoint     *points,
 			  gint		npoints);
   void (*draw_text)      (GdkDrawable  *drawable,
@@ -134,18 +134,18 @@ struct _GdkDrawableClass
                                           gint        *composite_x_offset,
                                           gint        *composite_y_offset);
 
-  void         (*_draw_pixbuf) (GdkDrawable *drawable,
-			        GdkGC       *gc,
-			        GdkPixbuf   *pixbuf,
-			        gint         src_x,
-			        gint         src_y,
-			        gint         dest_x,
-			        gint         dest_y,
-			        gint         width,
-			        gint         height,
-			        GdkRgbDither dither,
-			        gint         x_dither,
-			        gint         y_dither);
+  void         (*draw_pixbuf) (GdkDrawable *drawable,
+			       GdkGC       *gc,
+			       GdkPixbuf   *pixbuf,
+			       gint         src_x,
+			       gint         src_y,
+			       gint         dest_x,
+			       gint         dest_y,
+			       gint         width,
+			       gint         height,
+			       GdkRgbDither dither,
+			       gint         x_dither,
+			       gint         y_dither);
   GdkImage*    (*_copy_to_image) (GdkDrawable    *drawable,
 				  GdkImage       *image,
 				  gint            src_x,
@@ -211,20 +211,20 @@ void gdk_draw_point     (GdkDrawable      *drawable,
 			 gint              y);
 void gdk_draw_line      (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 gint              x1,
-			 gint              y1,
-			 gint              x2,
-			 gint              y2);
+			 gint              x1_,
+			 gint              y1_,
+			 gint              x2_,
+			 gint              y2_);
 void gdk_draw_rectangle (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 gint              filled,
+			 gboolean          filled,
 			 gint              x,
 			 gint              y,
 			 gint              width,
 			 gint              height);
 void gdk_draw_arc       (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 gint              filled,
+			 gboolean          filled,
 			 gint              x,
 			 gint              y,
 			 gint              width,
@@ -233,16 +233,20 @@ void gdk_draw_arc       (GdkDrawable      *drawable,
 			 gint              angle2);
 void gdk_draw_polygon   (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 gint              filled,
+			 gboolean          filled,
 			 GdkPoint         *points,
 			 gint              npoints);
-#ifndef GDK_DISABLE_DEPRECATED
+#if !defined (GDK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
+/* Used by gtk_default_draw_string () */
 void gdk_draw_string    (GdkDrawable      *drawable,
 			 GdkFont          *font,
 			 GdkGC            *gc,
 			 gint              x,
 			 gint              y,
 			 const gchar      *string);
+#endif /* !GDK_DISABLE_DEPRECATED || GTK_COMPILATION */
+#if !defined (GDK_DISABLE_DEPRECATED) || defined (GDK_COMPILATION)
+/* Used by gdk_pixmap_draw_text (), gdk_window_draw_text() */
 void gdk_draw_text      (GdkDrawable      *drawable,
 			 GdkFont          *font,
 			 GdkGC            *gc,
@@ -250,6 +254,7 @@ void gdk_draw_text      (GdkDrawable      *drawable,
 			 gint              y,
 			 const gchar      *text,
 			 gint              text_length);
+/* Used by gdk_pixmap_draw_text_wc (), gdk_window_draw_text_wc () */
 void gdk_draw_text_wc   (GdkDrawable      *drawable,
 			 GdkFont          *font,
 			 GdkGC            *gc,
@@ -257,7 +262,7 @@ void gdk_draw_text_wc   (GdkDrawable      *drawable,
 			 gint              y,
 			 const GdkWChar   *text,
 			 gint              text_length);
-#endif /* GDK_DISABLE_DEPRECATED */
+#endif /* !GDK_DISABLE_DEPRECATED || GDK_COMPILATION */
 void gdk_draw_drawable  (GdkDrawable      *drawable,
 			 GdkGC            *gc,
 			 GdkDrawable      *src,
@@ -288,6 +293,18 @@ void gdk_draw_lines     (GdkDrawable      *drawable,
 			 GdkGC            *gc,
 			 GdkPoint         *points,
 			 gint              npoints);
+void gdk_draw_pixbuf    (GdkDrawable      *drawable,
+			 GdkGC            *gc,
+			 GdkPixbuf        *pixbuf,
+			 gint              src_x,
+			 gint              src_y,
+			 gint              dest_x,
+			 gint              dest_y,
+			 gint              width,
+			 gint              height,
+			 GdkRgbDither      dither,
+			 gint              x_dither,
+			 gint              y_dither);
 
 void gdk_draw_glyphs      (GdkDrawable      *drawable,
 			   GdkGC            *gc,
@@ -326,11 +343,19 @@ void gdk_draw_layout_with_colors      (GdkDrawable     *drawable,
 #define gdk_draw_bitmap                gdk_draw_drawable
 #endif /* GDK_DISABLE_DEPRECATED */
 
-GdkImage* gdk_drawable_get_image (GdkDrawable *drawable,
-                                  gint         x,
-                                  gint         y,
-                                  gint         width,
-                                  gint         height);
+GdkImage* gdk_drawable_get_image      (GdkDrawable *drawable,
+                                       gint         x,
+                                       gint         y,
+                                       gint         width,
+                                       gint         height);
+GdkImage *gdk_drawable_copy_to_image (GdkDrawable  *drawable,
+				      GdkImage     *image,
+				      gint          src_x,
+				      gint          src_y,
+				      gint          dest_x,
+				      gint          dest_y,
+				      gint          width,
+				      gint          height);
 
 GdkRegion *gdk_drawable_get_clip_region    (GdkDrawable *drawable);
 GdkRegion *gdk_drawable_get_visible_region (GdkDrawable *drawable);
