@@ -97,6 +97,7 @@ _gdk_windowing_init (gint    *argc,
   GDK_NOTE (MISC, g_print ("Windows version: %08x\n", (guint) _windows_version));
 
   _gdk_input_locale = GetKeyboardLayout (0);
+  _gdk_input_locale_is_ime = ImmIsIME (_gdk_input_locale);
   GetLocaleInfo (MAKELCID (LOWORD (_gdk_input_locale), SORT_DEFAULT),
 		 LOCALE_IDEFAULTANSICODEPAGE,
 		 buf, sizeof (buf));
@@ -875,6 +876,11 @@ _gdk_win32_message_to_string (UINT msg)
       CASE (WM_PENWINFIRST);
       CASE (WM_PENWINLAST);
       CASE (WM_APP);
+#ifdef HAVE_WINTAB
+      CASE (WT_PACKET);
+      CASE (WT_CSRCHANGE);
+      CASE (WT_PROXIMITY);
+#endif
 #undef CASE
     default:
       if (msg >= WM_HANDHELDFIRST && msg <= WM_HANDHELDLAST)
@@ -922,18 +928,16 @@ _gdk_win32_gdkregion_to_string (const GdkRegion *rgn)
 gchar *
 _gdk_win32_drawable_description (GdkDrawable *d)
 {
-  GdkVisual *v;
-  gint width, height;
+  gint width, height, depth;
 
   gdk_drawable_get_size (d, &width, &height);
+  depth = gdk_drawable_get_depth (d);
 
   return static_printf
     ("%s:%p:%dx%dx%d",
      G_OBJECT_TYPE_NAME (d),
      GDK_DRAWABLE_HANDLE (d),
-     width, height,
-     (GDK_IS_PIXMAP (d) ? GDK_PIXMAP_IMPL_WIN32 (GDK_PIXMAP_OBJECT (d)->impl)->image->depth
-      : ((v = gdk_drawable_get_visual (d)) ? v->depth : gdk_visual_get_system ()->depth)));
+     width, height, depth);
 }
 
 #endif /* G_ENABLE_DEBUG */
