@@ -41,9 +41,7 @@
 
 #include <config.h>
 
-#if HAVE_XFT
 #include <X11/extensions/Xrender.h>
-#endif
 
 #define GDK_TYPE_GC_X11              (_gdk_gc_x11_get_type ())
 #define GDK_GC_X11(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_GC_X11, GdkGCX11))
@@ -67,10 +65,8 @@ struct _GdkGCX11
   guint16 dirty_mask;
   guint have_clip_mask : 1;
 
-#ifdef HAVE_XFT  
   Picture fg_picture;
   XRenderColor fg_picture_color; 
-#endif  
   gulong fg_pixel;
 };
 
@@ -106,12 +102,10 @@ gint _gdk_send_xevent      (GdkDisplay *display,
 
 GType _gdk_gc_x11_get_type (void);
 
-#ifdef HAVE_XFT
 gboolean _gdk_x11_have_render         (GdkDisplay *display);
 Picture  _gdk_x11_gc_get_fg_picture   (GdkGC      *gc);
 void     _gdk_gc_x11_get_fg_xft_color (GdkGC      *gc,
 				       XftColor   *xftcolor);
-#endif /* HAVE_XFT */
 
 GdkGC *_gdk_x11_gc_new                  (GdkDrawable     *drawable,
 					 GdkGCValues     *values,
@@ -167,10 +161,10 @@ void _gdk_xgrab_check_destroy (GdkWindow *window);
 gboolean _gdk_x11_display_is_root_window (GdkDisplay *display,
 					  Window      xroot_window);
 
-void _gdk_x11_register_event_type (GdkDisplay *display,
-				   gint        event_base,
- 				   gint        n_events);
-  
+void _gdk_x11_precache_atoms (GdkDisplay          *display,
+			      const gchar * const *atom_names,
+			      gint                 n_atoms);
+
 void _gdk_x11_events_init_screen   (GdkScreen *screen);
 void _gdk_x11_events_uninit_screen (GdkScreen *screen);
 
