@@ -64,29 +64,30 @@ static void gtk_button_box_get_child_property (GtkContainer      *container,
 #define DEFAULT_CHILD_IPAD_X 4
 #define DEFAULT_CHILD_IPAD_Y 0
 
-GtkType
+GType
 gtk_button_box_get_type (void)
 {
-  static GtkType button_box_type = 0;
+  static GType button_box_type = 0;
 
   if (!button_box_type)
     {
       static const GTypeInfo button_box_info =
       {
 	sizeof (GtkButtonBoxClass),
-	NULL,            /* base_init */
-	NULL,            /* base_finalize */
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
 	(GClassInitFunc) gtk_button_box_class_init,
-	NULL,            /* class_finalize */
-	NULL,            /* class_data */
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
 	sizeof (GtkButtonBox),
 	0,               /* n_preallocs */
 	(GInstanceInitFunc) gtk_button_box_init,
-	NULL,            /* value_table */
+	NULL,		/* value_table */
       };
 
-      button_box_type = g_type_register_static (GTK_TYPE_BOX, "GtkButtonBox",
-						&button_box_info, G_TYPE_FLAG_ABSTRACT);
+      button_box_type =
+	g_type_register_static (GTK_TYPE_BOX, "GtkButtonBox",
+				&button_box_info, G_TYPE_FLAG_ABSTRACT);
     }
 
   return button_box_type;
@@ -115,8 +116,8 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("child_min_width",
-							     _("Minimum child width"),
-							     _("Minimum width of buttons inside the box"),
+							     P_("Minimum child width"),
+							     P_("Minimum width of buttons inside the box"),
 							     0,
 							     G_MAXINT,
                                                              DEFAULT_CHILD_MIN_WIDTH,
@@ -124,8 +125,8 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("child_min_height",
-							     _("Minimum child height"),
-							     _("Minimum height of buttons inside the box"),
+							     P_("Minimum child height"),
+							     P_("Minimum height of buttons inside the box"),
 							     0,
 							     G_MAXINT,
                                                              DEFAULT_CHILD_MIN_HEIGHT,
@@ -133,8 +134,8 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("child_internal_pad_x",
-							     _("Child internal width padding"),
-							     _("Amount to increase child's size on either side"),
+							     P_("Child internal width padding"),
+							     P_("Amount to increase child's size on either side"),
 							     0,
 							     G_MAXINT,
                                                              DEFAULT_CHILD_IPAD_X,
@@ -142,8 +143,8 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("child_internal_pad_y",
-							     _("Child internal height padding"),
-							     _("Amount to increase child's size on the top and bottom"),
+							     P_("Child internal height padding"),
+							     P_("Amount to increase child's size on the top and bottom"),
 							     0,
 							     G_MAXINT,
                                                              DEFAULT_CHILD_IPAD_Y,
@@ -151,8 +152,8 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_LAYOUT_STYLE,
                                    g_param_spec_enum ("layout_style",
-                                                      _("Layout style"),
-                                                      _("How to layout the buttons in the box. Possible values are default, spread, edge, start and end"),
+                                                      P_("Layout style"),
+                                                      P_("How to layout the buttons in the box. Possible values are default, spread, edge, start and end"),
 						      GTK_TYPE_BUTTON_BOX_STYLE,
 						      GTK_BUTTONBOX_DEFAULT_STYLE,
                                                       G_PARAM_READWRITE));
@@ -160,8 +161,8 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_SECONDARY,
 					      g_param_spec_boolean ("secondary", 
-								    _("Secondary"),
-								    _("If TRUE, the child appears in a secondary group of children, suitable for, e.g., help buttons"),
+								    P_("Secondary"),
+								    P_("If TRUE, the child appears in a secondary group of children, suitable for, e.g., help buttons"),
 								    FALSE,
 								    G_PARAM_READWRITE));
 }
@@ -238,25 +239,12 @@ gtk_button_box_get_child_property (GtkContainer *container,
 				   GValue       *value,
 				   GParamSpec   *pspec)
 {
-  GList *list;
-  GtkBoxChild *child_info = NULL;
-
-  list = GTK_BOX (container)->children;
-  while (list)
-    {
-      child_info = list->data;
-      if (child_info->widget == child)
-	break;
-
-      list = list->next;
-    }
-
-  g_assert (list != NULL);
-  
   switch (property_id)
     {
     case CHILD_PROP_SECONDARY:
-      g_value_set_boolean (value, child_info->is_secondary);
+      g_value_set_boolean (value, 
+			   gtk_button_box_get_child_secondary (GTK_BUTTON_BOX (container), 
+							       child));
       break;
     default:
       GTK_CONTAINER_WARN_INVALID_CHILD_PROPERTY_ID (container, property_id, pspec);
@@ -337,6 +325,43 @@ gtk_button_box_get_layout (GtkButtonBox *widget)
   return widget->layout_style;
 }
 
+/**
+ * gtk_button_box_get_child_secondary:
+ * @widget: a #GtkButtonBox
+ * @child: a child of @widget 
+ * 
+ * Returns whether @child should appear in a secondary group of children.
+ *
+ * Return value: whether @child should appear in a secondary group of children.
+ *
+ * Since: 2.4
+ **/
+gboolean 
+gtk_button_box_get_child_secondary (GtkButtonBox *widget,
+				    GtkWidget    *child)
+{
+  GList *list;
+  GtkBoxChild *child_info;
+
+  g_return_val_if_fail (GTK_IS_BUTTON_BOX (widget), FALSE);
+  g_return_val_if_fail (GTK_IS_WIDGET (child), FALSE);
+
+  child_info = NULL;
+  list = GTK_BOX (widget)->children;
+  while (list)
+    {
+      child_info = list->data;
+      if (child_info->widget == child)
+	break;
+
+      list = list->next;
+    }
+
+  g_return_val_if_fail (list != NULL, FALSE);
+
+  return child_info->is_secondary;
+}
+
 /**
  * gtk_button_box_set_child_secondary
  * @widget: a #GtkButtonBox
