@@ -125,8 +125,8 @@ gtk_box_class_init (GtkBoxClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_SPACING,
                                    g_param_spec_int ("spacing",
-                                                     _("Spacing"),
-                                                     _("The amount of space between children"),
+                                                     P_("Spacing"),
+                                                     P_("The amount of space between children"),
                                                      0,
                                                      G_MAXINT,
                                                      0,
@@ -135,44 +135,44 @@ gtk_box_class_init (GtkBoxClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_HOMOGENEOUS,
                                    g_param_spec_boolean ("homogeneous",
-							 _("Homogeneous"),
-							 _("Whether the children should all be the same size"),
+							 P_("Homogeneous"),
+							 P_("Whether the children should all be the same size"),
 							 FALSE,
 							 G_PARAM_READABLE | G_PARAM_WRITABLE));
 
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_EXPAND,
 					      g_param_spec_boolean ("expand", 
-								    _("Expand"), 
-								    _("Whether the child should receive extra space when the parent grows"),
+								    P_("Expand"), 
+								    P_("Whether the child should receive extra space when the parent grows"),
 								    TRUE,
 								    G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_FILL,
 					      g_param_spec_boolean ("fill", 
-								    _("Fill"), 
-								    _("Whether extra space given to the child should be allocated to the child or used as padding"),
+								    P_("Fill"), 
+								    P_("Whether extra space given to the child should be allocated to the child or used as padding"),
 								    TRUE,
 								    G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_PADDING,
 					      g_param_spec_uint ("padding", 
-								 _("Padding"), 
-								 _("Extra space to put between the child and its neighbors, in pixels"),
+								 P_("Padding"), 
+								 P_("Extra space to put between the child and its neighbors, in pixels"),
 								 0, G_MAXINT, 0,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_PACK_TYPE,
 					      g_param_spec_enum ("pack_type", 
-								 _("Pack type"), 
-								 _("A GtkPackType indicating whether the child is packed with reference to the start or end of the parent"),
+								 P_("Pack type"), 
+								 P_("A GtkPackType indicating whether the child is packed with reference to the start or end of the parent"),
 								 GTK_TYPE_PACK_TYPE, GTK_PACK_START,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_POSITION,
 					      g_param_spec_int ("position", 
-								_("Position"), 
-								_("The index of the child in the parent"),
+								P_("Position"), 
+								P_("The index of the child in the parent"),
 								-1, G_MAXINT, 0,
 								G_PARAM_READWRITE));
 }
