@@ -30,7 +30,6 @@
 #include "gtklabel.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
 #include "gtkimage.h"
 #include "gtkhbox.h"
 #include "gtkstock.h"
@@ -39,8 +38,8 @@
 
 #define CHILD_SPACING     1
 
-static GtkBorder default_default_border = { 1, 1, 1, 1 };
-static GtkBorder default_default_outside_border = { 0, 0, 0, 0 };
+static const GtkBorder default_default_border = { 1, 1, 1, 1 };
+static const GtkBorder default_default_outside_border = { 0, 0, 0, 0 };
 
 /* Time out before giving up on getting a key release when animatng
  * the close button.
@@ -62,7 +61,19 @@ enum {
   PROP_LABEL,
   PROP_RELIEF,
   PROP_USE_UNDERLINE,
-  PROP_USE_STOCK
+  PROP_USE_STOCK,
+  PROP_FOCUS_ON_CLICK,
+  PROP_XALIGN,
+  PROP_YALIGN
+};
+
+#define GTK_BUTTON_GET_PRIVATE(o)       (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_BUTTON, GtkButtonPrivate))
+typedef struct _GtkButtonPrivate GtkButtonPrivate;
+
+struct _GtkButtonPrivate
+{
+  gfloat xalign;
+  gfloat yalign;
 };
 
 static void gtk_button_class_init     (GtkButtonClass   *klass);
@@ -98,9 +109,11 @@ static gint gtk_button_leave_notify   (GtkWidget        *widget,
 				       GdkEventCrossing *event);
 static void gtk_real_button_pressed   (GtkButton        *button);
 static void gtk_real_button_released  (GtkButton        *button);
-static void gtk_real_button_activate (GtkButton         *button);
+static void gtk_real_button_activate  (GtkButton         *button);
 static void gtk_button_update_state   (GtkButton        *button);
-static GtkType gtk_button_child_type  (GtkContainer     *container);
+static void gtk_button_add            (GtkContainer   *container,
+			               GtkWidget      *widget);
+static GType gtk_button_child_type    (GtkContainer     *container);
 static void gtk_button_finish_activate (GtkButton *button,
 					gboolean   do_it);
 
@@ -114,10 +127,10 @@ static GtkBinClass *parent_class = NULL;
 static guint button_signals[LAST_SIGNAL] = { 0 };
 
 
-GtkType
+GType
 gtk_button_get_type (void)
 {
-  static GtkType button_type = 0;
+  static GType button_type = 0;
 
   if (!button_type)
     {
@@ -134,7 +147,8 @@ gtk_button_get_type (void)
 	(GInstanceInitFunc) gtk_button_init,
       };
 
-      button_type = g_type_register_static (GTK_TYPE_BIN, "GtkButton", &button_info, 0);
+      button_type = g_type_register_static (GTK_TYPE_BIN, "GtkButton",
+					    &button_info, 0);
     }
 
   return button_type;
@@ -143,21 +157,21 @@ gtk_button_get_type (void)
 static void
 gtk_button_class_init (GtkButtonClass *klass)
 {
-  GObjectClass *g_object_class;
+  GObjectClass *gobject_class;
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
-  g_object_class = G_OBJECT_CLASS (klass);
+  gobject_class = G_OBJECT_CLASS (klass);
   object_class = (GtkObjectClass*) klass;
   widget_class = (GtkWidgetClass*) klass;
   container_class = (GtkContainerClass*) klass;
   
   parent_class = g_type_class_peek_parent (klass);
 
-  g_object_class->constructor = gtk_button_constructor;
-  g_object_class->set_property = gtk_button_set_property;
-  g_object_class->get_property = gtk_button_get_property;
+  gobject_class->constructor = gtk_button_constructor;
+  gobject_class->set_property = gtk_button_set_property;
+  gobject_class->get_property = gtk_button_get_property;
 
   object_class->destroy = gtk_button_destroy;
 
@@ -183,117 +197,182 @@ gtk_button_class_init (GtkButtonClass *klass)
   klass->leave = gtk_button_update_state;
   klass->activate = gtk_real_button_activate;
 
-  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+  g_object_class_install_property (gobject_class,
                                    PROP_LABEL,
                                    g_param_spec_string ("label",
-                                                        _("Label"),
-                                                        _("Text of the label widget inside the button, if the button contains a label widget."),
+                                                        P_("Label"),
+                                                        P_("Text of the label widget inside the button, if the button contains a label widget"),
                                                         NULL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
-  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+  g_object_class_install_property (gobject_class,
                                    PROP_USE_UNDERLINE,
                                    g_param_spec_boolean ("use_underline",
-							 _("Use underline"),
-							 _("If set, an underline in the text indicates the next character should be used for the mnemonic accelerator key"),
+							 P_("Use underline"),
+							 P_("If set, an underline in the text indicates the next character should be used for the mnemonic accelerator key"),
                                                         FALSE,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
-  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+  g_object_class_install_property (gobject_class,
                                    PROP_USE_STOCK,
                                    g_param_spec_boolean ("use_stock",
-							 _("Use stock"),
-							 _("If set, the label is used to pick a stock item instead of being displayed"),
+							 P_("Use stock"),
+							 P_("If set, the label is used to pick a stock item instead of being displayed"),
                                                         FALSE,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
-  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+  g_object_class_install_property (gobject_class,
+                                   PROP_FOCUS_ON_CLICK,
+                                   g_param_spec_boolean ("focus_on_click",
+							 P_("Focus on click"),
+							 P_("Whether the button grabs focus when it is clicked with the mouse"),
+							 TRUE,
+							 G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
                                    PROP_RELIEF,
                                    g_param_spec_enum ("relief",
-                                                      _("Border relief"),
-                                                      _("The border relief style."),
+                                                      P_("Border relief"),
+                                                      P_("The border relief style"),
                                                       GTK_TYPE_RELIEF_STYLE,
                                                       GTK_RELIEF_NORMAL,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
+  /**
+   * GtkButton:xalign:
+   *
+   * If the child of the button is a #GtkMisc or #GtkAlignment, this property 
+   * can be used to control it's horizontal alignment. 0.0 is left aligned, 
+   * 1.0 is right aligned.
+   * 
+   * Since: 2.4
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_XALIGN,
+                                   g_param_spec_float("xalign",
+                                                      P_("Horizontal alignment for child"),
+                                                      P_("Horizontal position of child in available space. 0.0 is left aligned, 1.0 is right aligned"),
+                                                      0.0,
+                                                      1.0,
+                                                      0.5,
+                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+
+  /**
+   * GtkButton:yalign:
+   *
+   * If the child of the button is a #GtkMisc or #GtkAlignment, this property 
+   * can be used to control it's vertical alignment. 0.0 is top aligned, 
+   * 1.0 is bottom aligned.
+   * 
+   * Since: 2.4
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_YALIGN,
+                                   g_param_spec_float("yalign",
+                                                      P_("Vertical alignment for child"),
+                                                      P_("Vertical position of child in available space. 0.0 is top aligned, 1.0 is bottom aligned"),
+                                                      0.0,
+                                                      1.0,
+                                                      0.5,
+                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
 
   button_signals[PRESSED] =
-    gtk_signal_new ("pressed",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkButtonClass, pressed),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("pressed",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkButtonClass, pressed),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   button_signals[RELEASED] =
-    gtk_signal_new ("released",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkButtonClass, released),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("released",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkButtonClass, released),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   button_signals[CLICKED] =
-    gtk_signal_new ("clicked",
-                    GTK_RUN_FIRST | GTK_RUN_ACTION,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkButtonClass, clicked),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("clicked",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkButtonClass, clicked),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   button_signals[ENTER] =
-    gtk_signal_new ("enter",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkButtonClass, enter),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("enter",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkButtonClass, enter),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   button_signals[LEAVE] =
-    gtk_signal_new ("leave",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkButtonClass, leave),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("leave",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkButtonClass, leave),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
+
+  /**
+   * GtkButton::activate:
+   *
+   * The "activate" signal on GtkButton is an action signal and
+   * emitting it causes the button to animate press then release. 
+   * Applications should never connect to this signal, but use the
+   * "clicked" signal.
+   */
   button_signals[ACTIVATE] =
-    gtk_signal_new ("activate",
-                    GTK_RUN_FIRST | GTK_RUN_ACTION,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkButtonClass, activate),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("activate",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkButtonClass, activate),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   widget_class->activate_signal = button_signals[ACTIVATE];
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_boxed ("default_border",
-							       _("Default Spacing"),
-							       _("Extra space to add for CAN_DEFAULT buttons"),
+							       P_("Default Spacing"),
+							       P_("Extra space to add for CAN_DEFAULT buttons"),
 							       GTK_TYPE_BORDER,
 							       G_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_boxed ("default_outside_border",
-							       _("Default Outside Spacing"),
-							       _("Extra space to add for CAN_DEFAULT buttons that is always drawn outside the border"),
+							       P_("Default Outside Spacing"),
+							       P_("Extra space to add for CAN_DEFAULT buttons that is always drawn outside the border"),
 							       GTK_TYPE_BORDER,
 							       G_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("child_displacement_x",
-							     _("Child X Displacement"),
-							     _("How far in the x direction to move the child when the button is depressed"),
+							     P_("Child X Displacement"),
+							     P_("How far in the x direction to move the child when the button is depressed"),
 							     G_MININT,
 							     G_MAXINT,
 							     0,
 							     G_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("child_displacement_y",
-							     _("Child Y Displacement"),
-							     _("How far in the y direction to move the child when the button is depressed"),
+							     P_("Child Y Displacement"),
+							     P_("How far in the y direction to move the child when the button is depressed"),
 							     G_MININT,
 							     G_MAXINT,
 							     0,
 							     G_PARAM_READABLE));
+
+  g_type_class_add_private (gobject_class, sizeof (GtkButtonPrivate));  
 }
 
 static void
 gtk_button_init (GtkButton *button)
 {
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
   GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 
@@ -307,6 +386,10 @@ gtk_button_init (GtkButton *button)
   button->use_underline = FALSE;
   button->depressed = FALSE;
   button->depress_on_activate = TRUE;
+  button->focus_on_click = TRUE;
+
+  priv->xalign = 0.5;
+  priv->yalign = 0.5;
 }
 
 static void
@@ -345,13 +428,41 @@ gtk_button_constructor (GType                  type,
 }
 
 
-static GtkType
+static GType
 gtk_button_child_type  (GtkContainer     *container)
 {
   if (!GTK_BIN (container)->child)
     return GTK_TYPE_WIDGET;
   else
-    return GTK_TYPE_NONE;
+    return G_TYPE_NONE;
+}
+
+static void
+maybe_set_alignment (GtkWidget *widget,
+		     gfloat     xalign,
+		     gfloat     yalign)
+{
+  if (GTK_IS_MISC (widget))
+    {
+      GtkMisc *misc = GTK_MISC (widget);
+      gtk_misc_set_alignment (misc, xalign, yalign);
+    }
+  else if (GTK_IS_ALIGNMENT (widget))
+    {
+      GtkAlignment *alignment = GTK_ALIGNMENT (widget);
+      gtk_alignment_set (alignment, xalign, yalign, alignment->xscale, alignment->yscale);
+    }
+}
+
+static void
+gtk_button_add (GtkContainer *container,
+		GtkWidget    *widget)
+{
+  GtkButton *button = GTK_BUTTON (container);
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+
+  maybe_set_alignment (widget, priv->xalign, priv->yalign);
+  GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
 }
 
 static void
@@ -360,9 +471,8 @@ gtk_button_set_property (GObject         *object,
                          const GValue    *value,
                          GParamSpec      *pspec)
 {
-  GtkButton *button;
-
-  button = GTK_BUTTON (object);
+  GtkButton *button = GTK_BUTTON (object);
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
 
   switch (prop_id)
     {
@@ -378,6 +488,15 @@ gtk_button_set_property (GObject         *object,
     case PROP_USE_STOCK:
       gtk_button_set_use_stock (button, g_value_get_boolean (value));
       break;
+    case PROP_FOCUS_ON_CLICK:
+      gtk_button_set_focus_on_click (button, g_value_get_boolean (value));
+      break;
+    case PROP_XALIGN:
+      gtk_button_set_alignment (button, g_value_get_float (value), priv->yalign);
+      break;
+    case PROP_YALIGN:
+      gtk_button_set_alignment (button, priv->xalign, g_value_get_float (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -390,9 +509,8 @@ gtk_button_get_property (GObject         *object,
                          GValue          *value,
                          GParamSpec      *pspec)
 {
-  GtkButton *button;
-
-  button = GTK_BUTTON (object);
+  GtkButton *button = GTK_BUTTON (object);
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
 
   switch (prop_id)
     {
@@ -408,6 +526,15 @@ gtk_button_get_property (GObject         *object,
     case PROP_USE_STOCK:
       g_value_set_boolean (value, button->use_stock);
       break;
+    case PROP_FOCUS_ON_CLICK:
+      g_value_set_boolean (value, button->focus_on_click);
+      break;
+    case PROP_XALIGN:
+      g_value_set_float (value, priv->xalign);
+      break;
+    case PROP_YALIGN:
+      g_value_set_float (value, priv->yalign);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -417,7 +544,7 @@ gtk_button_get_property (GObject         *object,
 GtkWidget*
 gtk_button_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_button_get_type ()));
+  return g_object_new (GTK_TYPE_BUTTON, NULL);
 }
 
 static void
@@ -428,7 +555,8 @@ gtk_button_construct_child (GtkButton *button)
   GtkWidget *image;
   GtkWidget *hbox;
   GtkWidget *align;
-
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+  
   if (!button->constructed)
     return;
   
@@ -438,7 +566,6 @@ gtk_button_construct_child (GtkButton *button)
   if (GTK_BIN (button)->child)
     gtk_container_remove (GTK_CONTAINER (button),
 			  GTK_BIN (button)->child);
-
   
   if (button->use_stock &&
       gtk_stock_lookup (button->label_text, &item))
@@ -450,7 +577,7 @@ gtk_button_construct_child (GtkButton *button)
       image = gtk_image_new_from_stock (button->label_text, GTK_ICON_SIZE_BUTTON);
       hbox = gtk_hbox_new (FALSE, 2);
 
-      align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+      align = gtk_alignment_new (priv->xalign, priv->yalign, 0.0, 0.0);
       
       gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -470,7 +597,7 @@ gtk_button_construct_child (GtkButton *button)
   else
     label = gtk_label_new (button->label_text);
   
-  gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (label), priv->xalign, priv->yalign);
 
   gtk_widget_show (label);
   gtk_container_add (GTK_CONTAINER (button), label);
@@ -530,7 +657,7 @@ gtk_button_pressed (GtkButton *button)
 {
   g_return_if_fail (GTK_IS_BUTTON (button));
 
-  gtk_signal_emit (GTK_OBJECT (button), button_signals[PRESSED]);
+  g_signal_emit (button, button_signals[PRESSED], 0);
 }
 
 void
@@ -538,7 +665,7 @@ gtk_button_released (GtkButton *button)
 {
   g_return_if_fail (GTK_IS_BUTTON (button));
 
-  gtk_signal_emit (GTK_OBJECT (button), button_signals[RELEASED]);
+  g_signal_emit (button, button_signals[RELEASED], 0);
 }
 
 void
@@ -546,7 +673,7 @@ gtk_button_clicked (GtkButton *button)
 {
   g_return_if_fail (GTK_IS_BUTTON (button));
 
-  gtk_signal_emit (GTK_OBJECT (button), button_signals[CLICKED]);
+  g_signal_emit (button, button_signals[CLICKED], 0);
 }
 
 void
@@ -554,7 +681,7 @@ gtk_button_enter (GtkButton *button)
 {
   g_return_if_fail (GTK_IS_BUTTON (button));
 
-  gtk_signal_emit (GTK_OBJECT (button), button_signals[ENTER]);
+  g_signal_emit (button, button_signals[ENTER], 0);
 }
 
 void
@@ -562,7 +689,7 @@ gtk_button_leave (GtkButton *button)
 {
   g_return_if_fail (GTK_IS_BUTTON (button));
 
-  gtk_signal_emit (GTK_OBJECT (button), button_signals[LEAVE]);
+  g_signal_emit (button, button_signals[LEAVE], 0);
 }
 
 void
@@ -608,8 +735,7 @@ gtk_button_realize (GtkWidget *widget)
   attributes.height = widget->allocation.height - border_width * 2;
   attributes.wclass = GDK_INPUT_ONLY;
   attributes.event_mask = gtk_widget_get_events (widget);
-  attributes.event_mask |= (GDK_EXPOSURE_MASK |
-			    GDK_BUTTON_PRESS_MASK |
+  attributes.event_mask |= (GDK_BUTTON_PRESS_MASK |
 			    GDK_BUTTON_RELEASE_MASK |
 			    GDK_ENTER_NOTIFY_MASK |
 			    GDK_LEAVE_NOTIFY_MASK);
@@ -617,7 +743,7 @@ gtk_button_realize (GtkWidget *widget)
   attributes_mask = GDK_WA_X | GDK_WA_Y;
 
   widget->window = gtk_widget_get_parent_window (widget);
-  gdk_window_ref (widget->window);
+  g_object_ref (widget->window);
   
   button->event_window = gdk_window_new (gtk_widget_get_parent_window (widget),
 					 &attributes, attributes_mask);
@@ -932,7 +1058,7 @@ gtk_button_button_press (GtkWidget      *widget,
     {
       button = GTK_BUTTON (widget);
 
-      if (!GTK_WIDGET_HAS_FOCUS (widget))
+      if (button->focus_on_click && !GTK_WIDGET_HAS_FOCUS (widget))
 	gtk_widget_grab_focus (widget);
 
       if (event->button == 1)
@@ -1084,7 +1210,8 @@ gtk_button_finish_activate (GtkButton *button,
   g_source_remove (button->activate_timeout);
   button->activate_timeout = 0;
 
-  gdk_keyboard_ungrab (gtk_get_current_event_time ());
+  gdk_display_keyboard_ungrab (gtk_widget_get_display (widget),
+			       gtk_get_current_event_time ());
   gtk_grab_remove (widget);
 
   button->button_down = FALSE;
@@ -1111,10 +1238,13 @@ void
 gtk_button_set_label (GtkButton   *button,
 		      const gchar *label)
 {
-  g_return_if_fail (GTK_IS_BUTTON (button));
+  gchar *new_label;
   
+  g_return_if_fail (GTK_IS_BUTTON (button));
+
+  new_label = g_strdup (label);
   g_free (button->label_text);
-  button->label_text = g_strdup (label);
+  button->label_text = new_label;
   
   gtk_button_construct_child (button);
   
@@ -1126,13 +1256,13 @@ gtk_button_set_label (GtkButton   *button,
  * @button: a #GtkButton
  *
  * Fetches the text from the label of the button, as set by
- * gtk_button_set_label().  This string is owned by the widget 
- * and must not be modified or freed.  If the label text has not 
- * been set the return value  will be %NULL. This will be the 
+ * gtk_button_set_label(). If the label text has not 
+ * been set the return value will be %NULL. This will be the 
  * case if you create an empty button with gtk_button_new() to 
  * use as a container.
  *
- * Return value: the text of the label widget. 
+ * Return value: The text of the label widget. This string is owned
+ * by the widget and must not be modified or freed.
  **/
 G_CONST_RETURN gchar *
 gtk_button_get_label (GtkButton *button)
@@ -1230,6 +1360,117 @@ gtk_button_get_use_stock (GtkButton *button)
   return button->use_stock;
 }
 
+/**
+ * gtk_button_set_focus_on_click:
+ * @button: a #GtkButton
+ * @focus_on_click: whether the mouse grabs focus when clicked with the mouse
+ * 
+ * Sets whether the button will grab focus when it is clicked with the mouse.
+ * Making mouse clicks not grab focus is useful in places like toolbars where
+ * you don't want the keyboard focus removed from the main area of the
+ * application.
+ *
+ * Since: 2.4
+ **/
+void
+gtk_button_set_focus_on_click (GtkButton *button,
+			       gboolean   focus_on_click)
+{
+  g_return_if_fail (GTK_IS_BUTTON (button));
+  
+  focus_on_click = focus_on_click != FALSE;
+
+  if (button->focus_on_click != focus_on_click)
+    {
+      button->focus_on_click = focus_on_click;
+      
+      g_object_notify (G_OBJECT (button), "focus_on_click");
+    }
+}
+
+/**
+ * gtk_button_get_focus_on_click:
+ * @button: a #GtkButton
+ * 
+ * Returns whether the button grabs focus when it is clicked with the mouse.
+ * See gtk_button_set_focus_on_click().
+ *
+ * Return value: %TRUE if the button grabs focus when it is clicked with
+ *               the mouse.
+ *
+ * Since: 2.4
+ **/
+gboolean
+gtk_button_get_focus_on_click (GtkButton *button)
+{
+  g_return_val_if_fail (GTK_IS_BUTTON (button), FALSE);
+  
+  return button->focus_on_click;
+}
+
+/**
+ * gtk_button_set_alignment:
+ * @button: a #GtkButton
+ * @xalign: the horizontal position of the child, 0.0 is left aligned, 
+ *   1.0 is right aligned
+ * @yalign: the vertical position of the child, 0.0 is top aligned, 
+ *   1.0 is bottom aligned
+ *
+ * Sets the alignment of the child. This property has no effect unless 
+ * the child is a #GtkMisc or a #GtkAligment.
+ *
+ * Since: 2.4
+ */
+void
+gtk_button_set_alignment (GtkButton *button,
+			  gfloat     xalign,
+			  gfloat     yalign)
+{
+  GtkButtonPrivate *priv;
+
+  g_return_if_fail (GTK_IS_BUTTON (button));
+  
+  priv = GTK_BUTTON_GET_PRIVATE (button);
+
+  priv->xalign = xalign;
+  priv->yalign = yalign;
+
+  maybe_set_alignment (GTK_BIN (button)->child, xalign, yalign);
+
+  g_object_freeze_notify (G_OBJECT (button));
+  g_object_notify (G_OBJECT (button), "xalign");
+  g_object_notify (G_OBJECT (button), "yalign");
+  g_object_thaw_notify (G_OBJECT (button));
+}
+
+/**
+ * gtk_button_get_alignment:
+ * @button: a #GtkButton
+ * @xalign: return location for horizontal alignment
+ * @yalign: return location for vertical alignment
+ *
+ * Gets the alignment of the child in the button.
+ *
+ * Since: 2.4
+ */
+void
+gtk_button_get_alignment (GtkButton *button,
+			  gfloat    *xalign,
+			  gfloat    *yalign)
+{
+  GtkButtonPrivate *priv;
+
+  g_return_if_fail (GTK_IS_BUTTON (button));
+  
+  priv = GTK_BUTTON_GET_PRIVATE (button);
+ 
+  if (xalign) 
+    *xalign = priv->xalign;
+
+  if (yalign)
+    *yalign = priv->yalign;
+}
+
 /**
  * _gtk_button_set_depressed:
  * @button: a #GtkButton
