@@ -73,6 +73,14 @@ enum {
 
 static guint toggle_cell_signals[LAST_SIGNAL] = { 0 };
 
+#define GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_CELL_RENDERER_TOGGLE, GtkCellRendererTogglePrivate))
+
+typedef struct _GtkCellRendererTogglePrivate GtkCellRendererTogglePrivate;
+struct _GtkCellRendererTogglePrivate
+{
+  guint inconsistent : 1;
+};
+
 
 GType
 gtk_cell_renderer_toggle_get_type (void)
@@ -129,8 +137,8 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 				   PROP_ACTIVE,
 				   g_param_spec_boolean ("active",
-							 _("Toggle state"),
-							 _("The toggle state of the button"),
+							 P_("Toggle state"),
+							 P_("The toggle state of the button"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -138,8 +146,8 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 		                   PROP_INCONSISTENT,
 				   g_param_spec_boolean ("inconsistent",
-					                 _("Inconsistent state"),
-							 _("The inconsistent state of the button"),
+					                 P_("Inconsistent state"),
+							 P_("The inconsistent state of the button"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -147,8 +155,8 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 				   PROP_ACTIVATABLE,
 				   g_param_spec_boolean ("activatable",
-							 _("Activatable"),
-							 _("The toggle button can be activated"),
+							 P_("Activatable"),
+							 P_("The toggle button can be activated"),
 							 TRUE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -156,8 +164,8 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 				   PROP_RADIO,
 				   g_param_spec_boolean ("radio",
-							 _("Radio state"),
-							 _("Draw the toggle button as a radio button"),
+							 P_("Radio state"),
+							 P_("Draw the toggle button as a radio button"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -172,6 +180,8 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 		  _gtk_marshal_VOID__STRING,
 		  G_TYPE_NONE, 1,
 		  G_TYPE_STRING);
+
+  g_type_class_add_private (object_class, sizeof (GtkCellRendererTogglePrivate));
 }
 
 static void
@@ -181,6 +191,9 @@ gtk_cell_renderer_toggle_get_property (GObject     *object,
 				       GParamSpec  *pspec)
 {
   GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (object);
+  GtkCellRendererTogglePrivate *priv;
+
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
   
   switch (param_id)
     {
@@ -188,16 +201,7 @@ gtk_cell_renderer_toggle_get_property (GObject     *object,
       g_value_set_boolean (value, celltoggle->active);
       break;
     case PROP_INCONSISTENT:
-      {
-	/* Move out of here when more properties start to use the info
-	 * thing. I put it here to not affect performance, this property
-	 * is not going to be used much.
-	 */
-	GtkCellRendererInfo *cellinfo;
-	cellinfo = g_object_get_data (object, GTK_CELL_RENDERER_INFO_KEY);
-
-        g_value_set_boolean (value, cellinfo->inconsistent);
-      }
+      g_value_set_boolean (value, priv->inconsistent);
       break;
     case PROP_ACTIVATABLE:
       g_value_set_boolean (value, celltoggle->activatable);
@@ -219,6 +223,9 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
 				       GParamSpec   *pspec)
 {
   GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (object);
+  GtkCellRendererTogglePrivate *priv;
+
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
   
   switch (param_id)
     {
@@ -227,14 +234,8 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
       g_object_notify (G_OBJECT(object), "active");
       break;
     case PROP_INCONSISTENT:
-      {
-	/* read comment in _get_property */
-	GtkCellRendererInfo *cellinfo;
-	cellinfo = g_object_get_data (object, GTK_CELL_RENDERER_INFO_KEY);
-
-	cellinfo->inconsistent = g_value_get_boolean (value);
-	g_object_notify (G_OBJECT (object), "inconsistent");
-      }
+      priv->inconsistent = g_value_get_boolean (value);
+      g_object_notify (G_OBJECT (object), "inconsistent");
       break;
     case PROP_ACTIVATABLE:
       celltoggle->activatable = g_value_get_boolean (value);
@@ -316,12 +317,14 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
 				 GtkCellRendererState  flags)
 {
   GtkCellRendererToggle *celltoggle = (GtkCellRendererToggle *) cell;
-  GtkCellRendererInfo *cellinfo;
+  GtkCellRendererTogglePrivate *priv;
   gint width, height;
   gint x_offset, y_offset;
   GtkShadowType shadow;
   GtkStateType state = 0;
 
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (cell);
+
   gtk_cell_renderer_toggle_get_size (cell, widget, cell_area,
 				     &x_offset, &y_offset,
 				     &width, &height);
@@ -331,9 +334,7 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
   if (width <= 0 || height <= 0)
     return;
 
-  cellinfo = g_object_get_data (G_OBJECT (cell), GTK_CELL_RENDERER_INFO_KEY);
-
-  if (cellinfo->inconsistent)
+  if (priv->inconsistent)
     shadow = GTK_SHADOW_ETCHED_IN;
   else
     shadow = celltoggle->active ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
