@@ -313,8 +313,10 @@ color_sample_drop_handle (GtkWidget        *widget,
   if (selection_data->length < 0)
     return;
   
-  if ((selection_data->format != 16) ||
-      (selection_data->length != 8))
+  /* We accept drops with the wrong format, since the KDE color
+   * chooser incorrectly drops application/x-color with format 8.
+   */
+  if (selection_data->length != 8)
     {
       g_warning ("Received invalid color data\n");
       return;
@@ -607,12 +609,11 @@ palette_paint (GtkWidget    *drawing_area,
 {
   if (drawing_area->window == NULL)
     return;
-  
-  gdk_window_clear_area (drawing_area->window,
-			 area->x,
-			 area->y,
-			 area->width, 
-			 area->height);
+
+  gdk_draw_rectangle (drawing_area->window,
+                      drawing_area->style->bg_gc[GTK_STATE_NORMAL],
+                      TRUE,
+                      area->x, area->y, area->width, area->height);
   
   if (GTK_WIDGET_HAS_FOCUS (drawing_area))
     {
@@ -709,7 +710,7 @@ get_current_colors (GtkColorSelection *colorsel)
   gchar *palette;
 
   settings = gtk_widget_get_settings (GTK_WIDGET (colorsel));
-  g_object_get (G_OBJECT (settings),
+  g_object_get (settings,
 		"gtk-color-palette", &palette,
 		NULL);
   
@@ -1023,8 +1024,10 @@ palette_drop_handle (GtkWidget        *widget,
   if (selection_data->length < 0)
     return;
   
-  if ((selection_data->format != 16) ||
-      (selection_data->length != 8))
+  /* We accept drops with the wrong format, since the KDE color
+   * chooser incorrectly drops application/x-color with format 8.
+   */
+  if (selection_data->length != 8)
     {
       g_warning ("Received invalid color data\n");
       return;
@@ -1375,9 +1378,9 @@ hex_changed (GtkWidget *hex_entry,
   text = gtk_editable_get_chars (GTK_EDITABLE (priv->hex_entry), 0, -1);
   if (gdk_color_parse (text, &color))
     {
-      priv->color[COLORSEL_RED] = CLAMP (color.red/65280.0, 0.0, 1.0);
-      priv->color[COLORSEL_GREEN] = CLAMP (color.green/65280.0, 0.0, 1.0);
-      priv->color[COLORSEL_BLUE] = CLAMP (color.blue/65280.0, 0.0, 1.0);
+      priv->color[COLORSEL_RED] = CLAMP (color.red/65535.0, 0.0, 1.0);
+      priv->color[COLORSEL_GREEN] = CLAMP (color.green/65535.0, 0.0, 1.0);
+      priv->color[COLORSEL_BLUE] = CLAMP (color.blue/65535.0, 0.0, 1.0);
       gtk_rgb_to_hsv (priv->color[COLORSEL_RED],
 		      priv->color[COLORSEL_GREEN],
 		      priv->color[COLORSEL_BLUE],
@@ -1389,6 +1392,16 @@ hex_changed (GtkWidget *hex_entry,
   g_free (text);
 }
 
+static gboolean
+hex_focus_out (GtkWidget     *hex_entry, 
+	       GdkEventFocus *event,
+	       gpointer       data)
+{
+  hex_changed (hex_entry, data);
+  
+  return FALSE;
+}
+
 static void
 hsv_changed (GtkWidget *hsv,
 	     gpointer   data)
@@ -1432,7 +1445,7 @@ adjustment_changed (GtkAdjustment *adjustment,
     {
     case COLORSEL_SATURATION:
     case COLORSEL_VALUE:
-      priv->color[GPOINTER_TO_INT (data)] = adjustment->value / 255;
+      priv->color[GPOINTER_TO_INT (data)] = adjustment->value / 100;
       gtk_hsv_to_rgb (priv->color[COLORSEL_HUE],
 		      priv->color[COLORSEL_SATURATION],
 		      priv->color[COLORSEL_VALUE],
@@ -1510,6 +1523,11 @@ make_label_spinbutton (GtkColorSelection *colorsel,
     {
       adjust = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 360.0, 1.0, 1.0, 1.0));
     }
+  else if (channel_type == COLORSEL_SATURATION ||
+	   channel_type == COLORSEL_VALUE)
+    {
+      adjust = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 1.0, 1.0));
+    }
   else
     {
       adjust = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 255.0, 1.0, 1.0, 1.0));
@@ -1586,10 +1604,10 @@ update_color (GtkColorSelection *colorsel)
 			    scale_round (priv->color[COLORSEL_HUE], 360));
   gtk_adjustment_set_value (gtk_spin_button_get_adjustment
 			    (GTK_SPIN_BUTTON (priv->sat_spinbutton)),
-			    scale_round (priv->color[COLORSEL_SATURATION], 255));
+			    scale_round (priv->color[COLORSEL_SATURATION], 100));
   gtk_adjustment_set_value (gtk_spin_button_get_adjustment
 			    (GTK_SPIN_BUTTON (priv->val_spinbutton)),
-			    scale_round (priv->color[COLORSEL_VALUE], 255));
+			    scale_round (priv->color[COLORSEL_VALUE], 100));
   gtk_adjustment_set_value (gtk_spin_button_get_adjustment
 			    (GTK_SPIN_BUTTON (priv->red_spinbutton)),
 			    scale_round (priv->color[COLORSEL_RED], 255));
@@ -1740,29 +1758,29 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_OPACITY_CONTROL,
                                    g_param_spec_boolean ("has_opacity_control",
-							 _("Has Opacity Control"),
-							 _("Whether the color selector should allow setting opacity"),
+							 P_("Has Opacity Control"),
+							 P_("Whether the color selector should allow setting opacity"),
 							 FALSE,
 							 G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_PALETTE,
                                    g_param_spec_boolean ("has_palette",
-							 _("Has palette"),
-							 _("Whether a palette should be used"),
+							 P_("Has palette"),
+							 P_("Whether a palette should be used"),
 							 FALSE,
 							 G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_CURRENT_COLOR,
                                    g_param_spec_boxed ("current_color",
-                                                       _("Current Color"),
-                                                       _("The current color"),
+                                                       P_("Current Color"),
+                                                       P_("The current color"),
                                                        GDK_TYPE_COLOR,
                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_CURRENT_ALPHA,
                                    g_param_spec_uint ("current_alpha",
-						      _("Current Alpha"),
-						      _("The current opacity value (0 fully transparent, 65535 fully opaque)"),
+						      P_("Current Alpha"),
+						      P_("The current opacity value (0 fully transparent, 65535 fully opaque)"),
 						      0, 65535, 65535,
 						      G_PARAM_READABLE | G_PARAM_WRITABLE));
   
@@ -1776,8 +1794,8 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
 		  G_TYPE_NONE, 0);
 
   gtk_settings_install_property (g_param_spec_string ("gtk-color-palette",
-                                                      _("Custom palette"),
-                                                      _("Palette to use in the color selector"),
+                                                      P_("Custom palette"),
+                                                      P_("Palette to use in the color selector"),
                                                       default_colors,
                                                       G_PARAM_READWRITE));
 }
@@ -1854,6 +1872,7 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   
   make_label_spinbutton (colorsel, &priv->hue_spinbutton, _("_Hue:"), table, 0, 0, COLORSEL_HUE,
                          _("Position on the color wheel."));
+  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (priv->hue_spinbutton), TRUE);
   make_label_spinbutton (colorsel, &priv->sat_spinbutton, _("_Saturation:"), table, 0, 1, COLORSEL_SATURATION,
                          _("\"Deepness\" of the color."));
   make_label_spinbutton (colorsel, &priv->val_spinbutton, _("_Value:"), table, 0, 2, COLORSEL_VALUE,
@@ -1874,7 +1893,7 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   priv->opacity_slider = gtk_hscale_new (adjust);
   gtk_tooltips_set_tip (priv->tooltips,
                         priv->opacity_slider,
-                        _("Transparency of the currently-selected color."), NULL);
+                        _("Transparency of the color."), NULL);
   gtk_label_set_mnemonic_widget (GTK_LABEL (priv->opacity_label),
                                  priv->opacity_slider);
   gtk_scale_set_draw_value (GTK_SCALE (priv->opacity_slider), FALSE);
@@ -1883,6 +1902,9 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
                     GINT_TO_POINTER (COLORSEL_OPACITY));
   gtk_table_attach_defaults (GTK_TABLE (table), priv->opacity_slider, 1, 7, 4, 5); 
   priv->opacity_entry = gtk_entry_new (); 
+  gtk_tooltips_set_tip (priv->tooltips,
+                        priv->opacity_entry,
+                        _("Transparency of the color."), NULL);
   gtk_widget_set_size_request (priv->opacity_entry, 40, -1); 
 
   g_signal_connect (priv->opacity_entry, "activate",
@@ -1899,6 +1921,9 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   g_signal_connect (priv->hex_entry, "activate",
                     G_CALLBACK (hex_changed), colorsel);
 
+  g_signal_connect (priv->hex_entry, "focus_out_event",
+                    G_CALLBACK (hex_focus_out), colorsel);
+
   gtk_tooltips_set_tip (priv->tooltips,
                         priv->hex_entry,
                         _("You can enter an HTML-style hexadecimal color value, or simply a color name such as 'orange' in this entry."), NULL);
@@ -2251,7 +2276,7 @@ gtk_color_selection_set_current_alpha (GtkColorSelection *colorsel,
  * Sets the current color to be @color.  The first time this is called, it will
  * also set the original color to be @color too.
  *
- * This function is deprecated, use gtk_color_selection_set_current_color() instead.
+ * Deprecated: Use gtk_color_selection_set_current_color() instead.
  **/
 void
 gtk_color_selection_set_color (GtkColorSelection    *colorsel,
@@ -2682,11 +2707,12 @@ gtk_color_selection_palette_to_string (const GdkColor *colors,
  * the new palette contents, and update the GtkSettings property
  * "gtk-color-palette" so all GtkColorSelection widgets will be modified.
  *
- * This function is deprecated in favor of
+ * Return value: the previous change palette hook (that was replaced).
+ *
+ * Deprecated: This function is deprecated in favor of 
  * gtk_color_selection_set_change_palette_with_screen_hook(), and does
  * not work in multihead environments.
  * 
- * Return value: the previous change palette hook (that was replaced).
  **/
 GtkColorSelectionChangePaletteFunc
 gtk_color_selection_set_change_palette_hook (GtkColorSelectionChangePaletteFunc func)
