@@ -34,6 +34,7 @@
 #include "gtkmarshalers.h"
 #include "gtkwindow.h"
 #include "gtkintl.h"
+#include "gtktoolbar.h"
 #include <gobject/gobjectnotifyqueue.c>
 #include <gobject/gvaluecollector.h>
 
@@ -199,16 +200,16 @@ gtk_container_class_init (GtkContainerClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_RESIZE_MODE,
                                    g_param_spec_enum ("resize_mode",
-                                                      _("Resize mode"),
-                                                      _("Specify how resize events are handled"),
+                                                      P_("Resize mode"),
+                                                      P_("Specify how resize events are handled"),
                                                       GTK_TYPE_RESIZE_MODE,
                                                       GTK_RESIZE_PARENT,
                                                       G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_BORDER_WIDTH,
                                    g_param_spec_uint ("border_width",
-                                                      _("Border width"),
-                                                      _("The width of the empty border outside the containers children"),
+                                                      P_("Border width"),
+                                                      P_("The width of the empty border outside the containers children"),
 						      0,
 						      G_MAXINT,
 						      0,
@@ -216,8 +217,8 @@ gtk_container_class_init (GtkContainerClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_CHILD,
                                    g_param_spec_object ("child",
-                                                      _("Child"),
-                                                      _("Can be used to add a new child to the container"),
+                                                      P_("Child"),
+                                                      P_("Can be used to add a new child to the container"),
                                                       GTK_TYPE_WIDGET,
 						      G_PARAM_WRITABLE));
   container_signals[ADD] =
@@ -981,7 +982,13 @@ gtk_container_remove (GtkContainer *container,
 {
   g_return_if_fail (GTK_IS_CONTAINER (container));
   g_return_if_fail (GTK_IS_WIDGET (widget));
-  g_return_if_fail (widget->parent == GTK_WIDGET (container));
+
+  /* When using the deprecated API of the toolbar, it is possible
+   * to legitimately call this function with a widget that is not
+   * a direct child of the container.
+   */
+  g_return_if_fail (GTK_IS_TOOLBAR (container) ||
+		    widget->parent == GTK_WIDGET (container));
   
   g_signal_emit (container, container_signals[REMOVE], 0, widget);
 }
@@ -1146,9 +1153,9 @@ _gtk_container_queue_resize (GtkContainer *container)
 		{
 		  GTK_PRIVATE_SET_FLAG (resize_container, GTK_RESIZE_PENDING);
 		  if (container_resize_queue == NULL)
-		    gtk_idle_add_priority (GTK_PRIORITY_RESIZE,
-					   gtk_container_idle_sizer,
-					   NULL);
+		    g_idle_add_full (GTK_PRIORITY_RESIZE,
+				     gtk_container_idle_sizer,
+				     NULL, NULL);
 		  container_resize_queue = g_slist_prepend (container_resize_queue, resize_container);
 		}
 	      break;
@@ -1563,10 +1570,12 @@ gtk_container_focus (GtkWidget        *widget,
 
 static gint
 tab_compare (gconstpointer a,
-	     gconstpointer b)
+	     gconstpointer b,
+	     gpointer      data)
 {
   const GtkWidget *child1 = a;
   const GtkWidget *child2 = b;
+  GtkTextDirection text_direction = GPOINTER_TO_INT (data);
 
   gint y1 = child1->allocation.y + child1->allocation.height / 2;
   gint y2 = child2->allocation.y + child2->allocation.height / 2;
@@ -1576,7 +1585,10 @@ tab_compare (gconstpointer a,
       gint x1 = child1->allocation.x + child1->allocation.width / 2;
       gint x2 = child2->allocation.x + child2->allocation.width / 2;
       
-      return (x1 < x2) ? -1 : ((x1 == x2) ? 0 : 1);
+      if (text_direction == GTK_TEXT_DIR_RTL) 
+	return (x1 < x2) ? 1 : ((x1 == x2) ? 0 : -1);
+      else
+	return (x1 < x2) ? -1 : ((x1 == x2) ? 0 : 1);
     }
   else
     return (y1 < y2) ? -1 : 1;
@@ -1588,7 +1600,8 @@ gtk_container_focus_sort_tab (GtkContainer     *container,
 			      GtkDirectionType  direction,
 			      GtkWidget        *old_focus)
 {
-  children = g_list_sort (children, tab_compare);
+  GtkTextDirection text_direction = gtk_widget_get_direction (GTK_WIDGET (container));
+  children = g_list_sort_with_data (children, tab_compare, GINT_TO_POINTER (text_direction));
 
   /* if we are going backwards then reverse the order
    *  of the children.
