@@ -84,6 +84,9 @@ static void     gtk_entry_completion_set_cell_data_func  (GtkCellLayout
                                                           GDestroyNotify           destroy);
 static void     gtk_entry_completion_clear_attributes    (GtkCellLayout           *cell_layout,
                                                           GtkCellRenderer         *cell);
+static void     gtk_entry_completion_reorder             (GtkCellLayout           *cell_layout,
+                                                          GtkCellRenderer         *cell,
+                                                          gint                     position);
 
 static gboolean gtk_entry_completion_visible_func        (GtkTreeModel            *model,
                                                           GtkTreeIter             *iter,
@@ -190,15 +193,15 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_MODEL,
                                    g_param_spec_object ("model",
-                                                        _("Completion Model"),
-                                                        _("The model to find matches in"),
+                                                        P_("Completion Model"),
+                                                        P_("The model to find matches in"),
                                                         GTK_TYPE_TREE_MODEL,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_MINIMUM_KEY_LENGTH,
                                    g_param_spec_int ("minimum_key_length",
-                                                     _("Minimum Key Length"),
-                                                     _("Minimum length of the search key in order to look up matches"),
+                                                     P_("Minimum Key Length"),
+                                                     P_("Minimum length of the search key in order to look up matches"),
                                                      -1,
                                                      G_MAXINT,
                                                      1,
@@ -216,6 +219,7 @@ gtk_entry_completion_cell_layout_init (GtkCellLayoutIface *iface)
   iface->add_attribute = gtk_entry_completion_add_attribute;
   iface->set_cell_data_func = gtk_entry_completion_set_cell_data_func;
   iface->clear_attributes = gtk_entry_completion_clear_attributes;
+  iface->reorder = gtk_entry_completion_reorder;
 }
 
 static void
@@ -258,6 +262,8 @@ gtk_entry_completion_init (GtkEntryCompletion *completion)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (priv->scrolled_window),
                                        GTK_SHADOW_ETCHED_IN);
 
+  /* a nasty hack to get the completions treeview to size nicely */
+  gtk_widget_set_size_request (GTK_SCROLLED_WINDOW (priv->scrolled_window)->vscrollbar, -1, 0);
 
   /* actions */
   priv->actions = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_BOOLEAN);
@@ -465,6 +471,20 @@ gtk_entry_completion_clear_attributes (GtkCellLayout   *cell_layout,
   gtk_tree_view_column_clear_attributes (priv->column, cell);
 }
 
+static void
+gtk_entry_completion_reorder (GtkCellLayout   *cell_layout,
+                              GtkCellRenderer *cell,
+                              gint             position)
+{
+  GtkEntryCompletionPrivate *priv;
+
+  g_return_if_fail (GTK_IS_ENTRY_COMPLETION (cell_layout));
+
+  priv = GTK_ENTRY_COMPLETION_GET_PRIVATE (cell_layout);
+
+  gtk_cell_layout_reorder (GTK_CELL_LAYOUT (priv->column), cell, position);
+}
+
 /* all those callbacks */
 static gboolean
 gtk_entry_completion_default_completion_func (GtkEntryCompletion *completion,
@@ -678,7 +698,8 @@ gtk_entry_completion_selection_changed (GtkTreeSelection *selection,
   if (completion->priv->first_sel_changed)
     {
       completion->priv->first_sel_changed = FALSE;
-      gtk_tree_selection_unselect_all (selection);
+      if (gtk_widget_is_focus (completion->priv->tree_view))
+        gtk_tree_selection_unselect_all (selection);
     }
 }
 
@@ -1029,12 +1050,57 @@ get_borders (GtkEntry *entry,
     }
 }
 
-/* this function is a bit nasty */
+/* some nasty size requisition */
+gint
+_gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
+{
+  gint items, height, x_border, y_border;
+
+  get_borders (GTK_ENTRY (completion->priv->entry), &x_border, &y_border);
+
+  items = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->filter_model), NULL);
+
+  items = MIN (items, 15);
+
+  gtk_tree_view_column_cell_get_size (completion->priv->column, NULL,
+                                      NULL, NULL, NULL, &height);
+
+  if (items <= 0)
+    gtk_widget_hide (completion->priv->scrolled_window);
+  else
+    gtk_widget_show (completion->priv->scrolled_window);
+
+  gtk_widget_set_size_request (completion->priv->tree_view,
+                               completion->priv->entry->allocation.width - 2 * x_border,
+                               items * height);
+
+  /* default on no match */
+  completion->priv->current_selected = -1;
+
+  items = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->actions), NULL);
+
+  if (items)
+    {
+      gtk_widget_show (completion->priv->action_view);
+
+      gtk_tree_view_column_cell_get_size (gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0),
+                                          NULL, NULL, NULL, NULL,
+                                          &height);
+
+      gtk_widget_set_size_request (completion->priv->action_view,
+                                   completion->priv->entry->allocation.width - 2 * x_border,
+                                   items * height);
+    }
+  else
+    gtk_widget_hide (completion->priv->action_view);
+
+  return height;
+}
+
 void
 _gtk_entry_completion_popup (GtkEntryCompletion *completion)
 {
   gint x, y, x_border, y_border;
-  gint items;
   gint height;
 
   if (GTK_WIDGET_MAPPED (completion->priv->popup_window))
@@ -1045,39 +1111,11 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
   gdk_window_get_origin (completion->priv->entry->window, &x, &y);
   get_borders (GTK_ENTRY (completion->priv->entry), &x_border, &y_border);
 
-  items = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->filter_model), NULL);
-
-  items = MIN (items, 15);
-
-  gtk_tree_view_column_cell_get_size (completion->priv->column, NULL,
-                                      NULL, NULL, NULL, &height);
-
-  gtk_widget_set_size_request (completion->priv->tree_view,
-                               completion->priv->entry->allocation.width - 2 * x_border,
-                               items * height);
-
-  if (items <= 0)
-    gtk_widget_hide (completion->priv->scrolled_window);
-
-  /* default on no match */
-  completion->priv->current_selected = -1;
-
-  items = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->actions), NULL);
-
-  if (items)
-    {
-      gtk_tree_view_column_cell_get_size (gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0),
-                                          NULL, NULL, NULL, NULL,
-                                          &height);
-
-      gtk_widget_set_size_request (completion->priv->action_view,
-                                   completion->priv->entry->allocation.width - 2 * x_border,
-                                   items * height);
-    }
-
   x += x_border;
   y += 2 * y_border;
 
+  height = _gtk_entry_completion_resize_popup (completion);
+
   gtk_window_move (GTK_WINDOW (completion->priv->popup_window), x, y + height);
 
   gtk_widget_show (completion->priv->popup_window);
