@@ -106,8 +106,8 @@ gtk_radio_button_class_init (GtkRadioButtonClass *class)
   g_object_class_install_property (gobject_class,
 				   PROP_GROUP,
 				   g_param_spec_object ("group",
-							_("Group"),
-							_("The radio button whose group this widget belongs."),
+							P_("Group"),
+							P_("The radio button whose group this widget belongs."),
 							GTK_TYPE_RADIO_BUTTON,
 							G_PARAM_WRITABLE));
   object_class->destroy = gtk_radio_button_destroy;
@@ -642,7 +642,7 @@ gtk_radio_button_draw_indicator (GtkCheckButton *check_button,
 	state_type = GTK_STATE_ACTIVE;
       else if (button->in_button)
 	state_type = GTK_STATE_PRELIGHT;
-      else if (!GTK_WIDGET_SENSITIVE (widget))
+      else if (!GTK_WIDGET_IS_SENSITIVE (widget))
 	state_type = GTK_STATE_INSENSITIVE;
       else
 	state_type = GTK_STATE_NORMAL;
