@@ -28,7 +28,6 @@
 #include "gtkbindings.h"
 #include "gtkmarshalers.h"
 #include "gtkscrolledwindow.h"
-#include "gtksignal.h"
 #include "gtkintl.h"
 
 
@@ -134,26 +133,29 @@ static GtkContainerClass *parent_class = NULL;
 
 static guint signals[LAST_SIGNAL] = {0};
 
-GtkType
+GType
 gtk_scrolled_window_get_type (void)
 {
-  static GtkType scrolled_window_type = 0;
+  static GType scrolled_window_type = 0;
 
   if (!scrolled_window_type)
     {
-      static const GtkTypeInfo scrolled_window_info =
+      static const GTypeInfo scrolled_window_info =
       {
-	"GtkScrolledWindow",
-	sizeof (GtkScrolledWindow),
 	sizeof (GtkScrolledWindowClass),
-	(GtkClassInitFunc) gtk_scrolled_window_class_init,
-	(GtkObjectInitFunc) gtk_scrolled_window_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_scrolled_window_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkScrolledWindow),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_scrolled_window_init,
       };
 
-      scrolled_window_type = gtk_type_unique (GTK_TYPE_BIN, &scrolled_window_info);
+      scrolled_window_type =
+	g_type_register_static (GTK_TYPE_BIN, "GtkScrolledWindow",
+				&scrolled_window_info, 0);
     }
 
   return scrolled_window_type;
@@ -203,7 +205,8 @@ gtk_scrolled_window_class_init (GtkScrolledWindowClass *class)
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   container_class = (GtkContainerClass*) class;
-  parent_class = gtk_type_class (GTK_TYPE_BIN);
+
+  parent_class = g_type_class_peek_parent (class);
 
   gobject_class->finalize = gtk_scrolled_window_finalize;
   gobject_class->set_property = gtk_scrolled_window_set_property;
@@ -229,30 +232,30 @@ gtk_scrolled_window_class_init (GtkScrolledWindowClass *class)
   g_object_class_install_property (gobject_class,
 				   PROP_HADJUSTMENT,
 				   g_param_spec_object ("hadjustment",
-							_("Horizontal Adjustment"),
-							_("The GtkAdjustment for the horizontal position."),
+							P_("Horizontal Adjustment"),
+							P_("The GtkAdjustment for the horizontal position"),
 							GTK_TYPE_ADJUSTMENT,
 							G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (gobject_class,
 				   PROP_VADJUSTMENT,
 				   g_param_spec_object ("vadjustment",
-							_("Vertical Adjustment"),
-							_("The GtkAdjustment for the vertical position."),
+							P_("Vertical Adjustment"),
+							P_("The GtkAdjustment for the vertical position"),
 							GTK_TYPE_ADJUSTMENT,
 							G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (gobject_class,
                                    PROP_HSCROLLBAR_POLICY,
                                    g_param_spec_enum ("hscrollbar_policy",
-                                                      _("Horizontal Scrollbar Policy"),
-                                                      _("When the horizontal scrollbar is displayed"),
+                                                      P_("Horizontal Scrollbar Policy"),
+                                                      P_("When the horizontal scrollbar is displayed"),
 						      GTK_TYPE_POLICY_TYPE,
 						      GTK_POLICY_ALWAYS,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_VSCROLLBAR_POLICY,
                                    g_param_spec_enum ("vscrollbar_policy",
-                                                      _("Vertical Scrollbar Policy"),
-                                                      _("When the vertical scrollbar is displayed"),
+                                                      P_("Vertical Scrollbar Policy"),
+                                                      P_("When the vertical scrollbar is displayed"),
 						      GTK_TYPE_POLICY_TYPE,
 						      GTK_POLICY_ALWAYS,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -260,20 +263,29 @@ gtk_scrolled_window_class_init (GtkScrolledWindowClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_WINDOW_PLACEMENT,
                                    g_param_spec_enum ("window_placement",
-                                                      _("Window Placement"),
-                                                      _("Where the contents are located with respect to the scrollbars"),
+                                                      P_("Window Placement"),
+                                                      P_("Where the contents are located with respect to the scrollbars"),
 						      GTK_TYPE_CORNER_TYPE,
 						      GTK_CORNER_TOP_LEFT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW_TYPE,
                                    g_param_spec_enum ("shadow_type",
-                                                      _("Shadow Type"),
-                                                      _("Style of bevel around the contents"),
+                                                      P_("Shadow Type"),
+                                                      P_("Style of bevel around the contents"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_NONE,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
 
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("scrollbar_spacing",
+							     P_("Scrollbar spacing"),
+							     P_("Number of pixels between the scrollbars and the scrolled window"),
+							     0,
+							     G_MAXINT,
+							     DEFAULT_SCROLLBAR_SPACING,
+							     G_PARAM_READABLE));
+
   signals[SCROLL_CHILD] =
     g_signal_new ("scroll_child",
                   G_TYPE_FROM_CLASS (object_class),
@@ -361,7 +373,7 @@ gtk_scrolled_window_set_hadjustment (GtkScrolledWindow *scrolled_window,
   if (hadjustment)
     g_return_if_fail (GTK_IS_ADJUSTMENT (hadjustment));
   else
-    hadjustment = (GtkAdjustment*) gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL);
+    hadjustment = (GtkAdjustment*) g_object_new (GTK_TYPE_ADJUSTMENT, NULL);
 
   bin = GTK_BIN (scrolled_window);
 
@@ -373,7 +385,7 @@ gtk_scrolled_window_set_hadjustment (GtkScrolledWindow *scrolled_window,
       gtk_widget_pop_composite_child ();
 
       gtk_widget_set_parent (scrolled_window->hscrollbar, GTK_WIDGET (scrolled_window));
-      gtk_widget_ref (scrolled_window->hscrollbar);
+      g_object_ref (scrolled_window->hscrollbar);
       gtk_widget_show (scrolled_window->hscrollbar);
     }
   else
@@ -384,17 +396,17 @@ gtk_scrolled_window_set_hadjustment (GtkScrolledWindow *scrolled_window,
       if (old_adjustment == hadjustment)
 	return;
 
-      gtk_signal_disconnect_by_func (GTK_OBJECT (old_adjustment),
-				     GTK_SIGNAL_FUNC (gtk_scrolled_window_adjustment_changed),
-				     scrolled_window);
+      g_signal_handlers_disconnect_by_func (old_adjustment,
+					    gtk_scrolled_window_adjustment_changed,
+					    scrolled_window);
       gtk_range_set_adjustment (GTK_RANGE (scrolled_window->hscrollbar),
 				hadjustment);
     }
   hadjustment = gtk_range_get_adjustment (GTK_RANGE (scrolled_window->hscrollbar));
-  gtk_signal_connect (GTK_OBJECT (hadjustment),
-		      "changed",
-		      GTK_SIGNAL_FUNC (gtk_scrolled_window_adjustment_changed),
-		      scrolled_window);
+  g_signal_connect (hadjustment,
+		    "changed",
+		    G_CALLBACK (gtk_scrolled_window_adjustment_changed),
+		    scrolled_window);
   gtk_scrolled_window_adjustment_changed (hadjustment, scrolled_window);
   
   if (bin->child)
@@ -415,7 +427,7 @@ gtk_scrolled_window_set_vadjustment (GtkScrolledWindow *scrolled_window,
   if (vadjustment)
     g_return_if_fail (GTK_IS_ADJUSTMENT (vadjustment));
   else
-    vadjustment = (GtkAdjustment*) gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL);
+    vadjustment = (GtkAdjustment*) g_object_new (GTK_TYPE_ADJUSTMENT, NULL);
 
   bin = GTK_BIN (scrolled_window);
 
@@ -427,7 +439,7 @@ gtk_scrolled_window_set_vadjustment (GtkScrolledWindow *scrolled_window,
       gtk_widget_pop_composite_child ();
 
       gtk_widget_set_parent (scrolled_window->vscrollbar, GTK_WIDGET (scrolled_window));
-      gtk_widget_ref (scrolled_window->vscrollbar);
+      g_object_ref (scrolled_window->vscrollbar);
       gtk_widget_show (scrolled_window->vscrollbar);
     }
   else
@@ -438,17 +450,17 @@ gtk_scrolled_window_set_vadjustment (GtkScrolledWindow *scrolled_window,
       if (old_adjustment == vadjustment)
 	return;
 
-      gtk_signal_disconnect_by_func (GTK_OBJECT (old_adjustment),
-				     GTK_SIGNAL_FUNC (gtk_scrolled_window_adjustment_changed),
-				     scrolled_window);
+      g_signal_handlers_disconnect_by_func (old_adjustment,
+					    gtk_scrolled_window_adjustment_changed,
+					    scrolled_window);
       gtk_range_set_adjustment (GTK_RANGE (scrolled_window->vscrollbar),
 				vadjustment);
     }
   vadjustment = gtk_range_get_adjustment (GTK_RANGE (scrolled_window->vscrollbar));
-  gtk_signal_connect (GTK_OBJECT (vadjustment),
-		      "changed",
-		      GTK_SIGNAL_FUNC (gtk_scrolled_window_adjustment_changed),
-		      scrolled_window);
+  g_signal_connect (vadjustment,
+		    "changed",
+		    G_CALLBACK (gtk_scrolled_window_adjustment_changed),
+		    scrolled_window);
   gtk_scrolled_window_adjustment_changed (vadjustment, scrolled_window);
 
   if (bin->child)
@@ -579,7 +591,7 @@ gtk_scrolled_window_set_shadow_type (GtkScrolledWindow *scrolled_window,
       scrolled_window->shadow_type = type;
 
       if (GTK_WIDGET_DRAWABLE (scrolled_window))
-	gtk_widget_queue_clear (GTK_WIDGET (scrolled_window));
+	gtk_widget_queue_draw (GTK_WIDGET (scrolled_window));
 
       gtk_widget_queue_resize (GTK_WIDGET (scrolled_window));
 
@@ -626,8 +638,8 @@ gtk_scrolled_window_finalize (GObject *object)
 {
   GtkScrolledWindow *scrolled_window = GTK_SCROLLED_WINDOW (object);
 
-  gtk_widget_unref (scrolled_window->hscrollbar);
-  gtk_widget_unref (scrolled_window->vscrollbar);
+  g_object_unref (scrolled_window->hscrollbar);
+  g_object_unref (scrolled_window->vscrollbar);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -891,7 +903,7 @@ gtk_scrolled_window_move_focus_out (GtkScrolledWindow *scrolled_window,
   g_object_ref (scrolled_window);
   
   scrolled_window->focus_out = TRUE;
-  g_signal_emit_by_name (G_OBJECT (toplevel), "move_focus", direction_type);
+  g_signal_emit_by_name (toplevel, "move_focus", direction_type);
   scrolled_window->focus_out = FALSE;
   
   g_object_unref (scrolled_window);
@@ -1017,11 +1029,18 @@ gtk_scrolled_window_relative_allocation (GtkWidget     *widget,
   if (scrolled_window->vscrollbar_visible)
     {
       GtkRequisition vscrollbar_requisition;
+      gboolean is_rtl;
+
       gtk_widget_get_child_requisition (scrolled_window->vscrollbar,
 					&vscrollbar_requisition);
+      is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
   
-      if (scrolled_window->window_placement == GTK_CORNER_TOP_RIGHT ||
-	  scrolled_window->window_placement == GTK_CORNER_BOTTOM_RIGHT)
+      if ((!is_rtl && 
+	   (scrolled_window->window_placement == GTK_CORNER_TOP_RIGHT ||
+	    scrolled_window->window_placement == GTK_CORNER_BOTTOM_RIGHT)) ||
+	  (is_rtl && 
+	   (scrolled_window->window_placement == GTK_CORNER_TOP_LEFT ||
+	    scrolled_window->window_placement == GTK_CORNER_BOTTOM_LEFT)))
 	allocation->x += (vscrollbar_requisition.width +  scrollbar_spacing);
 
       allocation->width = MAX (1, allocation->width - (vscrollbar_requisition.width + scrollbar_spacing));
@@ -1158,8 +1177,12 @@ gtk_scrolled_window_size_allocate (GtkWidget     *widget,
       gtk_widget_get_child_requisition (scrolled_window->vscrollbar,
 					&vscrollbar_requisition);
 
-      if (scrolled_window->window_placement == GTK_CORNER_TOP_LEFT ||
-	  scrolled_window->window_placement == GTK_CORNER_BOTTOM_LEFT)
+      if ((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL && 
+	   (scrolled_window->window_placement == GTK_CORNER_TOP_RIGHT ||
+	    scrolled_window->window_placement == GTK_CORNER_BOTTOM_RIGHT)) ||
+	  (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR && 
+	   (scrolled_window->window_placement == GTK_CORNER_TOP_LEFT ||
+	    scrolled_window->window_placement == GTK_CORNER_BOTTOM_LEFT)))
 	child_allocation.x = (relative_allocation.x +
 			      relative_allocation.width +
 			      scrollbar_spacing +
@@ -1381,6 +1404,16 @@ _gtk_scrolled_window_get_scrollbar_spacing (GtkScrolledWindow *scrolled_window)
 
   class = GTK_SCROLLED_WINDOW_GET_CLASS (scrolled_window);
 
-  return class->scrollbar_spacing >= 0 ? class->scrollbar_spacing : DEFAULT_SCROLLBAR_SPACING;
-}
+  if (class->scrollbar_spacing >= 0)
+    return class->scrollbar_spacing;
+  else
+    {
+      gint scrollbar_spacing;
+      
+      gtk_widget_style_get (GTK_WIDGET (scrolled_window),
+			    "scrollbar_spacing", &scrollbar_spacing,
+			    NULL);
 
+      return scrollbar_spacing;
+    }
+}
