@@ -239,6 +239,8 @@ status_style_change (GtkXIMInfo *info)
     info->status_style_setting = XIMStatusCallbacks;
   else if (status_style == GTK_IM_STATUS_NOTHING)
     info->status_style_setting = XIMStatusNothing;
+  else if (status_style == GTK_IM_STATUS_NONE)
+    info->status_style_setting = XIMStatusNone;
   else
     return;
 
@@ -258,6 +260,8 @@ preedit_style_change (GtkXIMInfo *info)
     info->preedit_style_setting = XIMPreeditCallbacks;
   else if (preedit_style == GTK_IM_PREEDIT_NOTHING)
     info->preedit_style_setting = XIMPreeditNothing;
+  else if (preedit_style == GTK_IM_PREEDIT_NONE)
+    info->preedit_style_setting = XIMPreeditNone;
   else
     return;
 
@@ -318,8 +322,8 @@ setup_im (GtkXIMInfo *info)
   if (!g_object_class_find_property (G_OBJECT_GET_CLASS (info->settings),
 				     "gtk-im-preedit-style"))
     gtk_settings_install_property (g_param_spec_enum ("gtk-im-preedit-style",
-						      _("IM Preedit style"),
-						      _("How to draw the input method preedit string"),
+						      P_("IM Preedit style"),
+						      P_("How to draw the input method preedit string"),
 						      GTK_TYPE_IM_PREEDIT_STYLE,
 						      GTK_IM_PREEDIT_CALLBACK,
 						      G_PARAM_READWRITE));
@@ -327,8 +331,8 @@ setup_im (GtkXIMInfo *info)
   if (!g_object_class_find_property (G_OBJECT_GET_CLASS (info->settings),
 				     "gtk-im-status-style"))
     gtk_settings_install_property (g_param_spec_enum ("gtk-im-status-style",
-						      _("IM Status style"),
-						      _("How to draw the input method statusbar"),
+						      P_("IM Status style"),
+						      P_("How to draw the input method statusbar"),
 						      GTK_TYPE_IM_STATUS_STYLE,
 						      GTK_IM_STATUS_CALLBACK,
 						      G_PARAM_READWRITE));
@@ -1272,7 +1276,7 @@ gtk_im_context_xim_get_ic (GtkIMContextXIM *context_xim)
 	   * stroke if both key pressed and released events are filtered.
 	   * (bugzilla #81759)
 	   */
-	  guint32 mask = 0;
+	  guint32 mask = 0xaaaaaaaa;
 	  XGetICValues (xic,
 			XNFilterEvents, &mask,
 			NULL);
